/*
ScheduleTogether Frontend
Copyright (C) 2024, Zervó Zadachin

DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License version 3
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License version 3 for more details.

This program is built in two major frameworks, and incorporates external libraries for certain functionalities.
These frameworks and libraries are covered by their respective licenses, and their usage
agreements are as outlined in their respective documentation or source
code.

This program uses assets and resources provided by third parties, and are likely not covered by this copyright notice. 
See their respective directories and relevant notices.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import { useAuthStore } from '@/stores'
import { getApiUrl } from './config-loader';

export const fetchWrapper = {
    get: request('GET'),
    post: request('POST'),
    put: request('PUT'),
    delete: request('DELETE')
}

function request(method: string) {
    return (url: string, body: object | undefined = undefined) => {
        const requestOptions: RequestInit = {
            body: undefined,
            method,
            headers: getAuthHeader(url) || {} as Record<string, string>
        };
        if (body) {
            (requestOptions.headers as Record<string, string>)['Content-Type'] = 'application/json';
            requestOptions.body = JSON.stringify(body);
        }
        return fetch(url, requestOptions).then(handleResponse);
    }
}

async function handleResponse(response: Response) {
    const isJson = response.headers?.get('content-type')?.includes('application/json');
    const data = isJson ? await response.json() : null;

    // check for error response
    if (!response.ok) {
        const { user } = useAuthStore();
        if ([401, 403].includes(response.status) && user) {
            // auto logout if 401 Unauthorized or 403 Forbidden response
            //logout();
        }

        // get error message from body or default to response status
        const error = (data && data.error) || response.status;
        return Promise.reject(error);
    }

    return data;
}

function getAuthHeader(url: string): Record<string, string> {
    const { auth } = useAuthStore();
    const isLoggedIn: boolean = !!auth?.token;
    const apiUrl: string = getApiUrl();
    const isApiUrl = url.startsWith(apiUrl);
    if (isLoggedIn && isApiUrl) {
        return { Authorization: `Bearer ${auth.token}`}
    } else {
        return {};
    }
}